/******************************************************************************************/
/*				                Fonctions de l'appli VTG_v3                               */
/******************************************************************************************/
// le 09 09 16 : ATmega164
#include "ATmega164_VMG.h"
#pragma interrupt_handler     IT_Conflit_SeaTalk:iv_EXT_INT2

extern void Init_USART0 (long int, char, char, char);   // USART0 pour reception NMEA 183
extern void Init_USART1 (long int, char, char, char);   // USAR1 pour emission SeaTalk
extern void Tx1_Data (char);
extern void Init_Tempo_T1_comp (void);                  // Timer1 pour tempo entre 2 emissions
extern void Start_Tempo_T1_comp (unsigned long int);
extern char Test_Tempo_T1_comp (void);
extern void Init_SPI (char);                      // SPI pour affichage LCD des donnees emises
extern void Init_Timer0_CTCA (void);              // Timer0 pour inversion de phase du LCD
extern void Tx_SPI (byte);                        // le LCD ne sert que pendant la mise au point

extern byte Polar;

void IT_Conflit_SeaTalk (void);
void Tempo_1s(void);
byte Emission_Trame_Seatalk (void);
byte Reception_Trame_NMEA(void);
void Initialisations (void);
char Pop_Tampon (void);
void Push_Tampon (char);

unsigned int Speed_Decimal;
byte Flg_INT2;
char Speed_CDUd[4];

// IT INT2 (sortie du XOR) TxD1 à '1' et bus Seatalk à '0'
void IT_Conflit_SeaTalk (void)
    {
    Flg_INT2 = 1;
    }

// Pile fifo empilage/depilage des caracters recus
// Pile FIFO de 64 octets (sinon modifier TAILLE)
// Avec registre d'etat : Etat_Tampon et 2 indices : I_Push, I_Pop
// Si la pile est pleine, Etat_Tampon = FULL
// Si la pile est vide, Etat_Tampon = EMPTY
char Tampon [TAILLE];
byte I_Push, I_Pop, Etat_Tampon = EMPTY;

// Fonction d'empilage
void Push_Tampon (char data)
	{
	Tampon [I_Push] = data;
	I_Push++; I_Push%=TAILLE;
	if (I_Push == I_Pop)
		Etat_Tampon = FULL;
	else
		Etat_Tampon = 0;
	}

// Fonction de dépilage
char Pop_Tampon (void)
	{
	char data;
	data =  Tampon [I_Pop];
	I_Pop++; I_Pop%=TAILLE;
	if (I_Push == I_Pop)
		Etat_Tampon = EMPTY;
	else
		Etat_Tampon = 0;
	return (data);
	}

// Initialisaton de l'appli VMG_v3
void Initialisations (void)
	{
	Init_USART0 (4800, 8, 'n', 1);
	UCSR0B &= (1<<UCSZ02);				// RaZ UCSR0B sauf bit UCSZ02
	UCSR0B |= (1<<RXCIE0);              // RXEN0 reste à 0
	Init_USART1 (4800, 9, 'n', 1);
	UCSR1B &= (1<<UCSZ12);				// RaZ UCSR1B sauf bit UCSZ12, TXEN1 reste à 0
	EICRA = (1<<ISC21)+(1<<ISC20);		// configure external IT on INT2 as rising edge
	EIMSK = (1<<INT2);					// enable external IT on INT2 (Pb2)
	Init_Tempo_T1_comp ();
	Init_SPI (2);
	Init_Timer0_CTCA ();
	DDRD |= (1<<3);
	PORTD |= (1<<3);
	}

// Reception de la trame NMEA 'VTG'
byte Reception_Trame_NMEA(void)
	{
	byte k=0;
	char rx_data;
	char deb_trame_Id [] = "$GPVTG,";

	UCSR0B |= (1<<RXEN0);               // enable reception (avec l'IT associee)
	// attente de l'occurrence de debut de trame '$GPVTG,' par lecture
	// sequentielle de la pile fifo, laquelle est chargee dans l'IT de reception de l'USART0
	do
        {
        Polar ^= 0xFF;
        Tx_SPI (1^Polar);               // on affiche "----" sur le LCD entre 2 emissions
        Tx_SPI (1^Polar);               // sur SeaTalk
        Tx_SPI (1^Polar);
        Tx_SPI (1^Polar);
        while (Etat_Tampon == EMPTY);
 		rx_data = Pop_Tampon ();
		if (rx_data == deb_trame_Id [k])
			k++;
		else
			return FALSE;
		}
	while (k <7);
	// attente de l'occurence "M," par lecture sequentielle de la pile fifo,
	do
		{
		while (Etat_Tampon == EMPTY);		// si elle ne vient pas apres la reception
		rx_data = Pop_Tampon ();			// de 20 caractères -> break
		k++;
		if (k>22)
			return FALSE;
		}
	while (rx_data != 'M');					// dans la trame VTG, 'M,' precede la donnee
	while (Etat_Tampon == EMPTY);			// route fond
	rx_data = Pop_Tampon ();
	if (rx_data != ',')						// si le caractere suivant 'M' n'est pas ','
		return FALSE;						// on annule la trame
	k=0;
	do										// acquisition de la vitesse fond
		{
		while (Etat_Tampon == EMPTY);		// acquisition des digits en codes ASCII
		rx_data = Pop_Tampon ();			// de la route fond (4 digits maxi)
		if ((rx_data > 0x2F)&&(rx_data < 0x3A))
			{
            Speed_CDUd[k] = rx_data-0x30;   // conversion ASCII en chaine numerique
			k++;
			}
		if (k > 4)
			return FALSE;
		}
	while ((rx_data != ',')&&(k < 5));
// Nombre de 2 à 4 chiffres
	switch (k)                              // mise en forme de la vitesse
        {                                   // selon le nombre de caracteres recu
        case 2:
        Speed_CDUd[3] = Speed_CDUd[1];
        Speed_CDUd[2] = Speed_CDUd[0];
        Speed_CDUd[1] = 0;
        Speed_CDUd[0] = 0;
        break;
        case 3:
        Speed_CDUd[3] = Speed_CDUd[2];
        Speed_CDUd[2] = Speed_CDUd[1];
        Speed_CDUd[1] = Speed_CDUd[0];
        Speed_CDUd[0] = 0;
        }
    Speed_Decimal = Speed_CDUd[3]+(Speed_CDUd[2]*10)+(Speed_CDUd[1]*100)+(Speed_CDUd[0]*1000);
	UCSR0B &=~(1<<RXEN0);	                // disable la reception de USART0
	return TRUE;
	}

// Emission de la trame Seatalk
// Attendre que le bus soit libre
// puis transmettre la trame 'vitesse fond' sur le bus Seatalk
byte Emission_Trame_Seatalk (void)
	{
	static byte speed_MSB, speed_LSB;
	do
		{
		while (!(PINB & (1<<3)));			// attendre bus seatalk libre (idle)
		Start_Tempo_T1_comp (2100);			// tester si le bus est libre pdt 2100µs
		while (Test_Tempo_T1_comp () == FALSE);
		}
	while (!(PINB & (1<<3)));
	Flg_INT2 = 0;
	UCSR1B |= (1<<TXEN1);				// enable USART1 emission sans IT
	UCSR1B |= (1<<TXB81);				// bit 8 à '1' (debut de trame)
    Tx1_Data (0x20);                    // ID de la vitesse surface car la GA
	if (Flg_INT2)                       // ne lit pas la vitesse fond
        return FALSE;
	UCSR1B &=~(1<<TXB81);				// bit 8 à '0' (donnee = data)
	Tx1_Data (0x01);
	if (Flg_INT2)
        return FALSE;

    speed_LSB = Speed_Decimal&0xFF;
	Tx1_Data (speed_LSB);        		// LSB first
	if (Flg_INT2)
        return FALSE;

    speed_MSB = Speed_Decimal>>8;
	Tx1_Data (speed_MSB);
	if (Flg_INT2)
        return FALSE;
	UCSR1B &=~(1<<TXEN1);				// disable USART1 emission sans IT
	return TRUE;
	}

// Tempo 1 seconde entre 2 emissions
void Tempo_1s(void)
	{
	Start_Tempo_T1_comp (1000000);
    TIMSK0  |= (1<<OCIE0A);                // autorise affichage LCD
	while (Test_Tempo_T1_comp () == FALSE);
    TIMSK0  &=~(1<<OCIE0A);                // bloque affichage LCD
	}

